import 'dart:async';
import 'dart:typed_data';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_measure/view/vid_painter/vid_painter.dart';
import 'package:flutter/material.dart' hide Image, decodeImageFromList;
import 'package:vid/us/vid_us_image.dart';
import 'dart:ui';

import '../player/events.dart';

class CarotidPlayer extends StatefulWidget {
  const CarotidPlayer(
    this.carotidController,
    this.playerController, {
    Key? key,
  }) : super(key: key);

  final Measure3DViewController carotidController;
  final VidPlayerController playerController;

  @override
  State<StatefulWidget> createState() => _CarotidPlayerState();
}

class _CarotidPlayerState extends State<CarotidPlayer> {
  Image? image;
  int frameIndex = 0;
  List<double> curColorFilterMatrix = <double>[
    1, 0, 0, 0, 0, // red
    0, 1, 0, 0, 0, // green
    0, 0, 1, 0, 0, // blue
    0, 0, 0, 1, 0, // alpha
  ];
  @override
  void initState() {
    widget.playerController.eventHandler.addListener(onControllerEvent);
    loadFrame(widget.carotidController.image4Measure!);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CarotidPlayer oldWidget) {
    if (oldWidget.playerController != widget.playerController) {
      throw UnsupportedError("[VidTestPlayer] unsupport replace controller.");
    }
    if (oldWidget.carotidController != widget.carotidController) {
      throw UnsupportedError("[VidTestPlayer] unsupport replace controller.");
    }
    super.didUpdateWidget(oldWidget);
  }

  void onControllerEvent(Object sender, VidPlayerEvent e) {
    if (e is VidPlayerFilterChangeEvent) {
      onFilterMatrixChanged(e);
    }
  }

  void onFilterMatrixChanged(VidPlayerFilterChangeEvent e) {
    setState(() {
      curColorFilterMatrix = e.filterMatrix;
    });
  }

  void loadFrame(VidUsImage e) async {
    frameIndex = e.index;
    final oldImage = image;
    image = await _loadImage(e.imageData);
    oldImage?.dispose();
    setState(() {});
  }

  Future<Image> _loadImage(Uint8List bytes) async {
    final Completer<Image> completer = Completer<Image>();
    decodeVidFromList(bytes, (Image image) => completer.complete(image));
    return completer.future;
  }

  void decodeVidFromList(Uint8List bytes, ImageDecoderCallback callback) async {
    ImmutableBuffer.fromUint8List(bytes).then((ImmutableBuffer buffer) async {
      final ImageDescriptor descriptor = await ImageDescriptor.encoded(buffer);
      descriptor.instantiateCodec().then((Codec codec) {
        final Future<FrameInfo> frameInfo = codec.getNextFrame();
        codec.dispose();
        return frameInfo;
      }).then((FrameInfo frameInfo) {
        buffer.dispose();
        descriptor.dispose();
        return callback(frameInfo.image);
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return buildBox(context, buildFrameView(context));
  }

  Widget buildBox(BuildContext context, Widget child) {
    return Container(
      alignment: Alignment.center,
      child: child,
    );
  }

  Widget buildFrameView(BuildContext context) {
    if (image != null) {
      final size = MediaQuery.of(context).size;
      return RepaintBoundary(
        child: CustomPaint(
          painter: VidPainter(
              image: image!,
              frameIndex: frameIndex,
              colorFilterMatrix: curColorFilterMatrix,
              enableScale: false),
          isComplex: false, //是否为复杂图像（true会缓存）
          size: size,
        ),
      );
    } else {
      return Container();
    }
  }

  @override
  void dispose() {
    widget.playerController.eventHandler.removeListener(onControllerEvent);
    image?.dispose();
    super.dispose();
  }
}
